@@ -14,8 +14,8 @@
  * @param numOperators
  * @param isConventionalHexagonal
  */
-KPointLatticeGenerator::KPointLatticeGenerator(const double primVectorsArray[3][3],
-                                               const double conventionalVectorsArray[3][3],
+KPointLatticeGenerator::KPointLatticeGenerator(const double primVectorsArray[][3],
+                                               const double conventionalVectorsArray[][3],
                                                const int latticePointOperatorsArray[][3][3],
                                                const int numOperators,
                                                const bool isConventionalHexagonal) {
@@ -40,7 +40,8 @@ KPointLatticeGenerator::KPointLatticeGenerator(const double primVectorsArray[3][
       }
     }
   }
-    KPointLatticeGenerator(primVectors,conventionalVectors,latticePointOperators,isConventionalHexagonal);
+
+  initializer(primVectors, conventionalVectors, latticePointOperators, isConventionalHexagonal);
 }
 
 /**
@@ -51,10 +52,10 @@ KPointLatticeGenerator::KPointLatticeGenerator(const double primVectorsArray[3][
  * @param latticePointOperators
  * @param isConventionalHexagonal
  */
-KPointLatticeGenerator::KPointLatticeGenerator(Tensor<double> primVectors,
-                                               Tensor<double> conventionalVectors,
-                                               std::vector<Tensor<int>> latticePointOperators,
-                                               const bool isConventionalHexagonal) {
+void KPointLatticeGenerator::initializer(const Tensor<double>& primVectors,
+                                         const Tensor<double>& conventionalVectors,
+                                         const std::vector<Tensor<int>>& latticePointOperators,
+                                         const bool isConventionalHexagonal) {
 
   static const double hexagonal2DShifts[9][2] = {
       {0, 0},
@@ -87,6 +88,7 @@ KPointLatticeGenerator::KPointLatticeGenerator(Tensor<double> primVectors,
     m_Other2DShifts[i][1] = other2DShifts[i][1];
   }
 
+  // Copies of the array of operators are made.
   m_PointOperators3D = latticePointOperators;
   // Make it global to save the dynamic allocation;
   m_KPointOperators = std::vector<Tensor<int> >(m_PointOperators3D.size(), Tensor<int>());
@@ -105,11 +107,12 @@ KPointLatticeGenerator::KPointLatticeGenerator(Tensor<double> primVectors,
     int coord = static_cast<int>(MSMath::round(directFaceCenter[dimNum]));
     isLatticePoint &= (coord % 2 == 0);
   }
-  if (isLatticePoint) {
-    conventionalVectors[0] = MSMath::arrayDivide(faceCenter, 2);
-    conventionalVectors[1] = MSMath::arraySubtract(conventionalVectors[0], conventionalVectors[1]);
-  }
+
   m_ConventionalVectors = conventionalVectors;
+  if (isLatticePoint) {
+    m_ConventionalVectors[0] = MSMath::arrayDivide(faceCenter, 2);
+    m_ConventionalVectors[1] = MSMath::arraySubtract(m_ConventionalVectors[0], m_ConventionalVectors[1]);
+  }
 
   if (isConventionalHexagonal) {
     m_Shifts2D = m_Hexagonal2DShifts;
@@ -118,7 +121,7 @@ KPointLatticeGenerator::KPointLatticeGenerator(Tensor<double> primVectors,
   }
 
   Tensor<double> conventionalToPrim =
-      MSMath::matrixMultiply<double, double, double>(conventionalVectors, m_CartesianToPrim);
+      MSMath::matrixMultiply<double, double, double>(m_ConventionalVectors, m_CartesianToPrim);
   Tensor<double> primToConventional = MSMath::simpleInverse(conventionalToPrim);
 
   // Get the two-dimensional point operations.
@@ -175,8 +178,10 @@ KPointLatticeGenerator::KPointLatticeGenerator(Tensor<double> primVectors,
   }
 
   m_NumConventionalPrimCells = fabs(MSMath::determinant(conventionalToPrim)); // maxLayers;
-  m_MaxZDistance = MSMath::magnitude(conventionalVectors[2]) / m_NumConventionalPrimCells
-      * (isConventionalHexagonal ? 3 : 2);  // 3 is for rhombohedral in primitive hexagonal;
+  // 3 is for primitive rhombohedral unit cell represented in a hexagonal lattice as the
+  // corresponding conventional lattice.
+  m_MaxZDistance = MSMath::magnitude(m_ConventionalVectors[2]) / m_NumConventionalPrimCells
+      * (isConventionalHexagonal ? 3 : 2);
   m_PointOperators2D = ops2D;
   m_PointOperators2D.resize(keeperIndex);
 }
@@ -195,8 +200,8 @@ void KPointLatticeGenerator::useScaleFactor(int spaceGroupNum) {
 }
 
 /*
- * Wrapper function of the private getKPointLattice function to disable the scaleFactor for now,
- * but keep the option.
+ * Wrapper function of the private getKPointLattice function to deal with scaleFactor to avoid
+ * deep nested loops.
  */
 KPointLattice KPointLatticeGenerator::getKPointLattice(const double minDistance, const int minSize) {
 
